/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "propertiesdialog.h"

#include "propertymodel.h"
#include "ui_propertiesform.h"
#include "propertydelegate.h"

PropertiesDialog::PropertiesDialog (QWidget* parent, Qt::WFlags flags) :
        KDialog (parent, flags),
        m_form (0),
        m_model (0)
{
    QWidget * w = new QWidget (this);
    m_form = new Ui::propertiesForm;
    m_form->setupUi (w);

    m_form->comboBoxModifier->addItem ("final", PropertyModel::FINAL);
    m_form->comboBoxModifier->addItem ("virtual", PropertyModel::VIRTUAL);
    m_form->comboBoxModifier->addItem ("pure virtual", PropertyModel::PURE_VIRTUAL);
    m_form->comboBoxModifier->addItem ("inline", PropertyModel::INLINE);

    m_form->tableView->setItemDelegateForColumn (2, new AccessListDelegate (m_form->tableView));
    m_form->tableView->setItemDelegateForColumn (5, new AccessListDelegate (m_form->tableView));
    m_form->tableView->horizontalHeader()->setResizeMode (QHeaderView::Interactive);

    connect (m_form->buttonSelectAll, SIGNAL (clicked()), this, SLOT (selectAll()));
    connect (m_form->buttonDeselectAll, SIGNAL (clicked()), this, SLOT (deselectAll()));
    connect (m_form->checkBoxQtProp, SIGNAL (clicked (bool)), this, SLOT (createQtProperties (bool)));
    connect (m_form->checkBoxDocumentation, SIGNAL (clicked (bool)), this, SLOT (createDocumentation (bool)));
    connect (m_form->comboBoxModifier, SIGNAL (currentIndexChanged (int)), this, SLOT (changeModifier (int)));

    setMainWidget (w);
    resize (600,350);
}

PropertiesDialog::~PropertiesDialog()
{
    delete m_form;
}


void PropertiesDialog::setModel (PropertyModel* model)
{
    m_form->tableView->setModel (0);
    m_form->tableView->reset();

    m_model = model;

    if (!m_model->m_hasQObject)
    {
        m_form->checkBoxQtProp->setEnabled (false);
        m_form->checkBoxQtProp->setVisible (false);
        m_model->m_createQtProperties = false;
    }
    else
    {
        m_form->checkBoxQtProp->setEnabled (true);
        m_form->checkBoxQtProp->setVisible (true);
        m_model->m_createQtProperties = m_form->checkBoxQtProp->checkState() == Qt::Checked;
    }

    m_model->m_createDocumentation = m_form->checkBoxDocumentation->checkState() == Qt::Checked;

    m_model->setModifier ( (PropertyModel::Modifier) m_form->comboBoxModifier->itemData (m_form->comboBoxModifier->currentIndex()).toInt());

    m_form->tableView->setModel (m_model);
    m_form->tableView->reset();
    m_form->tableView->resizeColumnsToContents();
}

void PropertiesDialog::show (PropertyModel * model)
{
    setModel (model);
    KDialog::show();
}


void PropertiesDialog::deselectAll()
{
    for (int i = 0; i < m_model->m_nodes.size(); i++)
    {
        PropertyNode &node = m_model->m_nodes[i];
        if (node.canCreateReader())
            node.setCreateReader (false);
        if (node.canCreateWriter())
            node.setCreateWriter (false);
    }
    m_form->tableView->reset();
}


void PropertiesDialog::selectAll()
{
    for (int i = 0; i < m_model->m_nodes.size(); i++)
    {
        PropertyNode &node = m_model->m_nodes[i];
        if (node.canCreateReader())
            node.setCreateReader (true);
        if (node.canCreateWriter())
            node.setCreateWriter (true);
    }
    m_form->tableView->reset();
}

void PropertiesDialog::createQtProperties (bool create)
{
    m_model->m_createQtProperties = create;
}


void PropertiesDialog::createDocumentation (bool create)
{
    m_model->m_createDocumentation = create;
}

void PropertiesDialog::changeModifier (int index)
{
    m_model->setModifier ( (PropertyModel::Modifier) m_form->comboBoxModifier->itemData (index).toInt());
}


#include "propertiesdialog.moc"

// kate: indent-mode cstyle; space-indent on; indent-width 4; replace-tabs on;

