/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "propertymodel.h"

#include <QtGui/QColor>
#include <KMessageBox>
#include <KLocalizedString>

PropertyModel::PropertyModel (QObject* parent) : QAbstractTableModel (parent)
{
    Q_UNUSED (parent);
    m_hasQObject = false;
    m_createQtProperties = false;
    m_modifier = FINAL;
}


PropertyModel::~PropertyModel()
{

}


Qt::ItemFlags PropertyModel::flags (const QModelIndex& index) const
{
    if (!index.isValid())
        return Qt::ItemIsEnabled;

    const PropertyNode &n = m_nodes[index.row() ];
    switch (index.column())
    {
    case 1:
        if (n.canCreateReader())
            return QAbstractTableModel::flags (index) | Qt::ItemIsEnabled | Qt::ItemIsUserCheckable;
        else
            return Qt::ItemIsUserCheckable;
    case 2:
        if (n.canCreateReader())
            return QAbstractTableModel::flags (index) | Qt::ItemIsEditable;
        else
            return Qt::NoItemFlags;
    case 3:
        if (n.canCreateReader())
            QAbstractTableModel::flags (index) | Qt::ItemIsEnabled;
        else
            return Qt::NoItemFlags;
    case 4:
        if (n.canCreateWriter())
            return  QAbstractTableModel::flags (index) | Qt::ItemIsEnabled | Qt::ItemIsUserCheckable;
        else
            return  Qt::ItemIsUserCheckable;
    case 5:
        if (n.canCreateWriter())
            return QAbstractTableModel::flags (index) | Qt::ItemIsEditable;
        else
            return Qt::NoItemFlags;
    case 6:
        if (n.canCreateWriter())
            QAbstractTableModel::flags (index) | Qt::ItemIsEnabled;
        else
            return Qt::NoItemFlags;
    }

    return QAbstractTableModel::flags (index) | Qt::ItemIsEnabled;
}


QVariant PropertyModel::headerData (int section, Qt::Orientation orientation, int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
    {
        switch (section)
        {
        case 0:
            return i18n("Member");
            //case 1:
            //        return QString();
        case 2:
            return i18n("Reader Access");
        case 3:
            return i18n("Reader Name");
            //case 4:
            //        return QString();
        case 5:
            return i18n("Writer Access");
        case 6:
            return i18n("Writer Name");
        default:
            return QVariant();
        }
    }

    return QAbstractTableModel::headerData (section, orientation, role);
}


bool PropertyModel::setData (const QModelIndex& index, const QVariant& value, int role)
{
    if (index.row() > m_nodes.size()-1 || index.column() > columnCount())
        return false;

    PropertyNode &n = m_nodes[index.row() ];

    if (role == Qt::CheckStateRole)
    {
        switch (index.column())
        {
        case 1:
            n.setCreateReader (value.toInt() == Qt::Checked ? true : false);
            break;
        case 4:
            n.setCreateWriter (value.toInt() == Qt::Checked ? true : false);
            break;
        }
        return true;
    }

    if (role == Qt::EditRole)
    {
        switch (index.column())
        {
        case 2:
            n.setReaderAccess (value.toString());
            break;
        case 5:
            n.setWriterAccess (value.toString());
            break;
        }
        return true;
    }

    return false;
}

QVariant PropertyModel::data (const QModelIndex& index, int role) const
{
    if (index.row() > m_nodes.size() -1 || index.column() > columnCount())
        return QVariant();

    const PropertyNode &n = m_nodes[index.row() ];

    if (role == Qt::CheckStateRole)
    {
        switch (index.column())
        {
        case 1:
            return n.createReader() ? Qt::Checked : Qt::Unchecked;
        case 4:
            return n.createWriter() ? Qt::Checked : Qt::Unchecked;
        }
        return QVariant();
    }

    if (role == Qt::EditRole)
    {
        switch (index.column())
        {
        case 1:
            return n.createReader() ? Qt::Checked : Qt::Unchecked;
        case 2:
            return n.readerAccess();
        case 4:
            return n.createWriter() ? Qt::Checked : Qt::Unchecked;
        case 5:
            return n.writerAccess();
        }
        return QVariant();
    }

    if (role == Qt::DecorationRole)
    {
        switch (index.column())
        {
        case 0:
            return n.icon();
        }
        return QVariant();
    }

    if (role == Qt::ForegroundRole)
    {
        if (index.column() < 1)
            return QColor (128,128,128);
    }

    if (role == Qt::DisplayRole)
    {
        switch (index.column())
        {
        case 0:
            return n.name() + ":" + n.type();
        case 1:
            return QString();
        case 2:
            return n.readerAccess();
        case 3:
            return n.readerName();
        case 4:
            return QString();
        case 5:
            return n.writerAccess();
        case 6:
            return n.writerName();
        }
    }

    return QVariant();
}

#include "propertymodel.moc"

// kate: indent-mode cstyle; space-indent on; indent-width 4; replace-tabs on;
