/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef PROPERTYMODEL_H
#define PROPERTYMODEL_H

#include <QtCore/QAbstractTableModel>
#include <QtCore/QModelIndex>
#include <QtCore/QVariant>
#include <QtCore/QString>


#include "propertynode.h"

using namespace KDevelop;

namespace KDevelop
{
class ClassDeclaration ;
class ClassMemberDeclaration ;
}

class PropertiesDialog;

class PropertyModel : public QAbstractTableModel
{
    Q_OBJECT

    friend class PropertiesDialog;
public:

    PropertyModel (QObject *parent = 0);
    ~PropertyModel();

    enum Modifier {FINAL, VIRTUAL, PURE_VIRTUAL, INLINE};

    virtual int columnCount (const QModelIndex& parent = QModelIndex()) const
    {
        Q_UNUSED (parent);
        return 7;
    }

    virtual QVariant data (const QModelIndex& index, int role = Qt::DisplayRole) const;

    virtual bool setData (const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);

    virtual int rowCount (const QModelIndex& parent = QModelIndex()) const
    {
        Q_UNUSED (parent);
        return m_nodes.size();
    }

    virtual QVariant headerData (int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;


    virtual Qt::ItemFlags flags (const QModelIndex& index) const;

    virtual bool createQtProperties() const
    {
        return m_createQtProperties;
    }

    virtual bool hasQObject() const
    {
        return m_hasQObject;
    }

    const QList< PropertyNode >& nodes() const
    {
        return m_nodes;
    }

    void setNodes (const QList< PropertyNode > &n)
    {
        m_nodes = n;
    }

    void setHasQObject (bool hq)
    {
        m_hasQObject = hq;
    }

    void setCreateQtProperties (bool cqp)
    {
        m_createQtProperties = cqp;
    }

    ClassDeclaration* klass() const
    {
        return m_klass;
    }

    ClassMemberDeclaration* selectedMember() const
    {
        return m_selectedMember;
    }

    void setKlass (KDevelop::ClassDeclaration* k)
    {
        m_klass = k;
    }

    void setSelectedMember (KDevelop::ClassMemberDeclaration* sm)
    {
        m_selectedMember = sm;
    }

    Modifier modifier() const
    {
        return m_modifier;
    }

    void setModifier (Modifier m)
    {
        m_modifier = m;
    }

    bool createDocumentation() const
    {
        return m_createDocumentation;
    }

    void setCreateDocumentation (bool cd)
    {
        m_createDocumentation = cd;
    }

protected:
    QList<PropertyNode> m_nodes;
    bool m_hasQObject;
    bool m_createQtProperties;
    bool m_createDocumentation;
    Modifier m_modifier;
    ClassDeclaration *m_klass;
    ClassMemberDeclaration *m_selectedMember;
};

#endif // PROPERTYMODEL_H

// kate: indent-mode cstyle; space-indent on; indent-width 4; replace-tabs on;
